==> Prerelease-5.16.tar.gz: rejui/src/JUI/Widgets/CommandLine.cpp <==
#include <JUI/Widgets/CommandLine.hpp>

namespace JUI
{

    // Get current date/time, format is YYYY-MM-DD.HH:mm:ss
    const std::string currentDateTime() {
        time_t     now = time(0);
        struct tm  tstruct;
        char       buf[80];
        tstruct = *localtime(&now);
        // Visit http://en.cppreference.com/w/cpp/chrono/c/strftime
        // for more information about date/time format
        strftime(buf, sizeof(buf), "%Y-%m-%d.%X", &tstruct);

        return buf;
    }

    CommandLine::CommandLine() {
        this->SetTitle("Console");

        message_log_box = new JUI::ScrollingRect(this->ViewportInstance());
        message_log_box->Size(JUI::UDim2(0, -input_form_height, 1, 1));

        message_log_list = new JUI::VerticalListLayout(message_log_box);
        message_log_list->LayoutOrder(JUI::LayoutOrder::V::BOTTOM);

        input_box = new JUI::TextInputForm(this->ViewportInstance());

        input_box->SetAutoCompleteText(">Type Commands Here");
        input_box->SetContent("");
        input_box->Size(JUI::UDim2(0, 20, 1, 0));
        input_box->Position(JUI::UDim2(0, -20, 0, 1));
        input_box->BGColor(Colors::Grays::DarkSlateGray);
        input_box->SetTextSize(16);
        input_box->SetTextColor(Colors::White);
        input_box->SetAutocompleteTextColor(Colors::Gray);

        input_box->OnReturn += [this] (const std::string& msg) {
            OnInputBoxSend(msg);
            Log(std::format("[{}] >{}", currentDateTime(), msg));
        };
    }

    CommandLine::CommandLine(Widget *parent): CommandLine() {
        this->Parent(parent);
    }

    bool CommandLine::Opened() const { return this->open == true;}

    bool CommandLine::Closed() const { return this->open == false;}

    void CommandLine::Open(bool openVal) { SetOpen(openVal); }

    void CommandLine::Close(bool openVal) { SetOpen(openVal);}

    void CommandLine::SetOpen(bool openVal) {
        this->Visible(openVal);
        this->open = openVal;
    }

    void CommandLine::Log(const std::string &message, const Color4 &color) {
        const Color4 light {60, 60, 60};
        const Color4 dark {80, 80, 80};

        auto* entry = new JUI::TextRect(message_log_list);
        entry->Size({100_percent, UDim(message_height, 0)});
        entry->SetContent(message);
        entry->LayoutOrder(index++);
        entry->BGColor(index%2 == 0 ? light : dark);
        entry->SetTextColor(color);
        entry->SetTextSize(14);
    }

    bool CommandLine::ObserveKeyInput(Key key, bool pressed) {
        if (!IsVisible())
            return false;

        return Widget::ObserveKeyInput(key, pressed);
    }

    void CommandLine::OnInputBoxSend(const std::string &message) {
        OnInput.Invoke(message);
        input_box->SetContent("");
        input_box->SetAutoCompleteText("");
    }
}

==> Prerelease-6.3.tar.gz: rejui/src/JUI/Widgets/CommandLine.cpp <==
#include <JUI/Widgets/CommandLine.hpp>

namespace JUI
{

    // Get current date/time, format is YYYY-MM-DD.HH:mm:ss
    const std::string currentDateTime() {
        time_t     now = time(0);
        struct tm  tstruct;
        char       buf[80];
        tstruct = *localtime(&now);
        // Visit http://en.cppreference.com/w/cpp/chrono/c/strftime
        // for more information about date/time format
        strftime(buf, sizeof(buf), "%Y-%m-%d.%X", &tstruct);

        return buf;
    }

    CommandLine::CommandLine() {
        this->Title("Console");

        message_log_box = new JUI::ScrollingRect(this->ViewportInstance());
        message_log_box->Size(JUI::UDim2(0, -input_form_height, 1, 1));

        message_log_list = new JUI::VerticalListLayout(message_log_box);
        message_log_list->LayoutOrder(JUI::LayoutOrder::V::BOTTOM);

        input_box = new JUI::TextInputForm(this->ViewportInstance());

        input_box->SetAutoCompleteText(">Type Commands Here");
        input_box->Content("");
        input_box->Size(JUI::UDim2(0, 20, 1, 0));
        input_box->Position(JUI::UDim2(0, -20, 0, 1));
        input_box->BGColor(Colors::Grays::DarkSlateGray);
        input_box->TextSize(16);
        input_box->TextColor(Colors::White);
        input_box->SetAutocompleteTextColor(Colors::Gray);

        input_box->OnReturn += [this] (const std::string& msg) {
            OnInputBoxSend(msg);
            Log(std::format("[{}] >{}", currentDateTime(), msg));
        };
    }

    CommandLine::CommandLine(Widget *parent): CommandLine() {
        this->Parent(parent);
    }

    bool CommandLine::Opened() const { return this->open == true;}

    bool CommandLine::Closed() const { return this->open == false;}

    void CommandLine::Open(bool openVal) { SetOpen(openVal); }

    void CommandLine::Close(bool openVal) { SetOpen(openVal);}

    void CommandLine::SetOpen(bool openVal) {
        this->Visible(openVal);
        this->open = openVal;
    }

    void CommandLine::Log(const std::string &message, const Color4 &color) {
        const Color4 light {60, 60, 60};
        const Color4 dark {80, 80, 80};

        auto* entry = new JUI::TextRect(message_log_list);
        entry->Size({100_percent, UDim(message_height, 0)});
        entry->Content(message);
        entry->LayoutOrder(index++);
        entry->BGColor(index%2 == 0 ? light : dark);
        entry->TextColor(color);
        entry->TextSize(14);

        message_log_box->canvas_height += entry->GetAbsoluteSize().y;
    }

    bool CommandLine::ObserveKeyInput(Key key, bool pressed) {
        if (!IsVisible())
            return false;

        return Widget::ObserveKeyInput(key, pressed);
    }

    void CommandLine::OnInputBoxSend(const std::string &message) {
        OnInput.Invoke(message);
        input_box->Content("");
        input_box->SetAutoCompleteText("");
    }
}
